(* File absyn.ml *)
(* Stephanie Weirich, adapted from Amr Sabry *)

type var = string

type prim =     Plus    (* + *)
              | Times   (* * *)
              | Minus   (* - *)
              | Equal   (* = *)
              | And     (* and *)
              | Or      (* or  *)
              | Not     (* not *)
              | Zero    (* zero? *)
              | LT      (* < *)
              | GT      (* > *)
              | Add1    (* add1 *)
              | Sub1    (* sub1 *)
              | Hd      (* hd *)
              | Tl      (* tl *)
              | Cons    (* cons *)
              | IsNull  (* null? *)
              | Pair    (* pair *)
              | Fst     (* fst *)
              | Snd     (* snd *)

type typ =     IntType 
             | BoolType
             | ProcType of typ list * typ
             | ListType of typ
             | PairType of typ * typ
				 | VarType of string

(* We use a reference so that we can remember the new type name 
 * generated for this type annotation. *)
type annot = typ option ref
              
type exp =     Var     of var             
             | Lambda  of annot list * var list * exp  
             | App     of exp * exp list  
             | LitInt  of int             
             | LitBool of bool    
               (* [] not annotated with type anymore *)
             | LitNull 
             | Prim    of prim * exp list 
             | Let     of var list * exp list * exp 
             | If      of exp * exp * exp 
               (* Letrec holds: result types, proc names, arg types, arg ids, bodies, letrec body *)
             | Letrec  of annot list * var list * annot list list * var list list * exp list * exp           
             | Varassign of var * exp
             | Begin   of exp * exp list
             
	       
			
(********************************************************************)

